@@ -20,25 +20,55 @@
 /* First, include the header file for the plugin, to bring in the
  * object definition and other useful things.
  */
-#include "gstffmpegdec.h"
-#include "gstffmpegenc.h"
+
+#include "config.h"
+#include <gst/gst.h>
+#ifdef HAVE_FFMPEG_UNINSTALLED
+#include <avcodec.h>
+#include <avformat.h>
+#else
+#include <ffmpeg/avcodec.h>
+#include <ffmpeg/avformat.h>
+#endif
+
+extern gboolean gst_ffmpegdemux_register (GstPlugin *plugin);
+extern gboolean gst_ffmpegdec_register (GstPlugin *plugin);
+extern gboolean gst_ffmpegenc_register (GstPlugin *plugin);
+extern gboolean gst_ffmpegmux_register (GstPlugin *plugin);
+extern gboolean gst_ffmpegcsp_register (GstPlugin *plugin);
+	
+extern URLProtocol gstreamer_protocol;
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
+
   avcodec_init ();
   avcodec_register_all ();
+  av_register_all ();
 
   gst_ffmpegenc_register (plugin);
   gst_ffmpegdec_register (plugin);
+  /*gst_ffmpegdemux_register (plugin);*/
+  /*gst_ffmpegmux_register (plugin);*/
+  gst_ffmpegcsp_register (plugin);
+
+  /*register_protocol (&gstreamer_protocol);*/
 
   /* Now we can return the pointer to the newly created Plugin object. */
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "ffmpeg",
-  plugin_init
-};
+  "All FFMPEG codecs",
+  plugin_init,
+  FFMPEG_VERSION,
+  "LGPL",
+  "FFMpeg",
+  "http://ffmpeg.sourceforge.net/"
+)
