@@ -23,6 +23,7 @@
 
 #include "gstflacenc.h"
 #include "gstflacdec.h"
+#include "gstflactag.h"
 
 #include "flac_compat.h"
 
@@ -32,11 +33,17 @@ plugin_init (GstPlugin *plugin)
   if (!gst_library_load ("gstbytestream"))
     return FALSE;
 
+  /* we need the gsttags plugin for metadata querying */
+  if (!gst_plugin_load ("gsttags"))
+    return FALSE;
+
   if (!gst_element_register (plugin, "flacenc", GST_RANK_NONE, GST_TYPE_FLACENC))
     return FALSE;
 
   if (!gst_element_register (plugin, "flacdec", GST_RANK_PRIMARY, GST_TYPE_FLACDEC))
     return FALSE;
+  if (!gst_element_register (plugin, "flactag", GST_RANK_PRIMARY, gst_flac_tag_get_type ())) 
+    return FALSE;
   
   return TRUE;
 }
@@ -49,6 +56,5 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN)
