@@ -25,20 +25,18 @@
 #include <string.h>
 
 #include <gstflacenc.h>
-
+#include <gst/tag/tag.h>
+#include <gst/gsttaginterface.h>
 #include "flac_compat.h"
 
-extern GstPadTemplate *gst_flacenc_src_template, *gst_flacenc_sink_template;
+static GstPadTemplate *src_template, *sink_template;
 
 /* elementfactory information */
 GstElementDetails flacenc_details = {
   "FLAC encoder",
-  "Codec/Audio/Encoder",
-  "LGPL",
+  "Codec/Encoder/Audio",
   "Encodes audio with the FLAC lossless audio encoder",
-  VERSION,
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
 };
 
 /* FlacEnc signals and args */
@@ -64,6 +62,7 @@ enum {
   ARG_RICE_PARAMETER_SEARCH_DIST,
 };
 
+static void             gst_flacenc_base_init           (gpointer g_class);
 static void		gst_flacenc_init		(FlacEnc *flacenc);
 static void		gst_flacenc_class_init		(FlacEncClass *klass);
 static void 		gst_flacenc_dispose 		(GObject *object);
@@ -101,7 +100,7 @@ flacenc_get_type (void)
   if (!flacenc_type) {
     static const GTypeInfo flacenc_info = {
       sizeof(FlacEncClass),
-      NULL,
+      gst_flacenc_base_init,
       NULL,
       (GClassInitFunc)gst_flacenc_class_init,
       NULL,
@@ -110,7 +109,15 @@ flacenc_get_type (void)
       0,
       (GInstanceInitFunc)gst_flacenc_init,
     };
+
+    static const GInterfaceInfo tag_setter_info = {
+      NULL,
+      NULL,
+      NULL
+    };
+
     flacenc_type = g_type_register_static (GST_TYPE_ELEMENT, "FlacEnc", &flacenc_info, 0);
+    g_type_add_interface_static (flacenc_type, GST_TYPE_TAG_SETTER, &tag_setter_info);
   }
   return flacenc_type;
 }
@@ -169,6 +176,56 @@ gst_flacenc_quality_get_type (void)
   return qtype;
 }
 
+static GstCaps*
+flac_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_flac",
+  	"audio/x-flac",
+	/* gst_props_new (
+ 	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL) */NULL);
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_raw",
+  	"audio/x-raw-int",
+	gst_props_new (
+    	    "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
+    	    "signed", 		GST_PROPS_BOOLEAN (TRUE),
+    	    "width", 		GST_PROPS_INT (16),
+    	    "depth",    	GST_PROPS_INT (16),
+    	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL));
+}
+
+static void
+gst_flacenc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *flac_caps;
+
+  raw_caps = raw_caps_factory ();
+  flac_caps = flac_caps_factory ();
+
+  sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
+					GST_PAD_ALWAYS, 
+					raw_caps, NULL);
+  src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+				       GST_PAD_ALWAYS, 
+				       flac_caps, NULL);
+  gst_element_class_add_pad_template (element_class, sink_template);
+  gst_element_class_add_pad_template (element_class, src_template);
+  gst_element_class_set_details (element_class, &flacenc_details);
+}
+
 static void
 gst_flacenc_class_init (FlacEncClass *klass)
 {
@@ -266,50 +323,24 @@ gst_flacenc_class_init (FlacEncClass *klass)
 static void
 gst_flacenc_init (FlacEnc *flacenc)
 {
-  flacenc->sinkpad = gst_pad_new_from_template (gst_flacenc_sink_template, "sink");
+  flacenc->sinkpad = gst_pad_new_from_template (sink_template, "sink");
   gst_element_add_pad(GST_ELEMENT(flacenc),flacenc->sinkpad);
   gst_pad_set_chain_function(flacenc->sinkpad,gst_flacenc_chain);
   gst_pad_set_link_function (flacenc->sinkpad, gst_flacenc_sinkconnect);
 
-  flacenc->srcpad = gst_pad_new_from_template (gst_flacenc_src_template, "src");
+  flacenc->srcpad = gst_pad_new_from_template (src_template, "src");
   gst_element_add_pad(GST_ELEMENT(flacenc),flacenc->srcpad);
 
   GST_FLAG_SET (flacenc, GST_ELEMENT_EVENT_AWARE);
 
   flacenc->encoder = FLAC__seekable_stream_encoder_new();
 
-  FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
-		                        gst_flacenc_write_callback);
-  FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
-		                        gst_flacenc_seek_callback);
-  					
-  FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
-		                        flacenc);
-
-  flacenc->metadata = GST_CAPS_NEW (
-                  "flacenc_metadata",
-                  "application/x-gst-metadata",
-                    "DESCRIPTION",      GST_PROPS_STRING ("Track encoded with GStreamer"),
-                    "DATE",             GST_PROPS_STRING (""),
-                    "TRACKNUMBER",      GST_PROPS_STRING (""),
-                    "TITLE",            GST_PROPS_STRING (""),
-                    "ARTIST",           GST_PROPS_STRING (""),
-                    "ALBUM",            GST_PROPS_STRING (""),
-                    "GENRE",            GST_PROPS_STRING (""),
-                    "VERSION",          GST_PROPS_STRING (""),
-                    "COPYRIGHT",        GST_PROPS_STRING (""),
-                    "LICENSE",          GST_PROPS_STRING (""),
-                    "ORGANISATION",     GST_PROPS_STRING (""),
-                    "LOCATION",         GST_PROPS_STRING (""),
-                    "CONTACT",          GST_PROPS_STRING (""),
-                    "ISRC",             GST_PROPS_STRING ("")
-                  );
-		  
   flacenc->negotiated = FALSE;
   flacenc->first = TRUE;
   flacenc->first_buf = NULL;
   flacenc->data = NULL;
   gst_flacenc_update_quality (flacenc, DEFAULT_QUALITY);
+  flacenc->tags = gst_tag_list_new ();
 }
 
 static void
@@ -444,46 +475,51 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
   return FLAC__STREAM_ENCODER_OK;
 }
 
-static void
-gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
+static void 
+add_one_tag (const GstTagList *list, const gchar *tag, 
+	     gpointer user_data)
 {
-  guint indice;
-  guint comments;
-  const gchar *meta_types[] = { "TITLE", "VERSION", "ALBUM", "TRACKNUMBER",
-                                "ARTIST", "PERFORMER", "COPYRIGHT", "LICENSE",
-                                "ORGANISATION", "DESCRIPTION", "GENRE", "DATE",
-                                "LOCATION", "CONTACT", "ISRC", NULL };
+      GList *comments;
+      GList *it;
+      FlacEnc *flacenc = GST_FLACENC (user_data);
+
+      comments = gst_tag_to_vorbis_comments (list, tag);
+      for (it = comments; it != NULL; it = it->next) {
+	      FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
+	      commment_entry.length = GUINT32_TO_LE (strlen(it->data));
+	      commment_entry.entry  =  it->data;
+	      FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], 
+								  flacenc->meta[0]->data.vorbis_comment.num_comments, 
+								  commment_entry, 
+								  TRUE);
+	      g_free (it->data);
+      }
+      g_list_free (comments);
+}
+
+static void
+gst_flacenc_set_metadata (FlacEnc *flacenc)
+{
+  const GstTagList *user_tags;
+  GstTagList *copy;
 
   g_return_if_fail (flacenc != NULL);
-
+  user_tags = gst_tag_setter_get_list (GST_TAG_SETTER (flacenc));
+  if ((flacenc->tags == NULL) && (user_tags == NULL)) {
+    return;
+  }
+  copy = gst_tag_list_merge (user_tags, flacenc->tags, 
+			     gst_tag_setter_get_merge_mode (GST_TAG_SETTER (flacenc)));
   flacenc->meta = g_malloc (sizeof (FLAC__StreamMetadata **));
 
   flacenc->meta[0] = FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
-  
-  for ( indice = 0, comments=0; meta_types[indice] != NULL; indice++) {
-    if (gst_caps_has_property(caps, meta_types[indice])) {
-      const gchar *entry;
-      gchar *data;
-      FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
-      
-      gst_caps_get_string(caps, (gchar *)meta_types[indice], &entry);
-
-      if (!strcmp (entry, ""))
-	continue;
-   
-      data = g_strdup_printf("%s=%s", meta_types[indice], entry);
-      commment_entry.length = GUINT32_TO_LE (strlen(entry) + strlen(meta_types[indice]) + 1);
-      commment_entry.entry  =  g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
-
-      FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], comments++, commment_entry, TRUE);
-    }
-  }
+  gst_tag_list_foreach ((GstTagList*)copy, add_one_tag, flacenc);
 
   FLAC__seekable_stream_encoder_set_metadata(flacenc->encoder, flacenc->meta, 1);
+  gst_tag_list_free (copy);
 }
 
 
-
 static void
 gst_flacenc_chain (GstPad *pad, GstData *_data)
 {
@@ -505,10 +541,19 @@ gst_flacenc_chain (GstPad *pad, GstData *_data)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
 	FLAC__seekable_stream_encoder_finish(flacenc->encoder);
+	break;
+      case GST_EVENT_TAG:
+	if (flacenc->tags) {
+	  gst_tag_list_merge (flacenc->tags, gst_event_tag_get_list (event), 
+		  gst_tag_setter_get_merge_mode (GST_TAG_SETTER (flacenc)));
+	} else {
+	  g_assert_not_reached ();
+	}
+	break;
       default:
-	gst_pad_event_default (pad, event);
         break;
     }
+    gst_pad_event_default (pad, event);
     return;
   }
 
@@ -528,7 +573,15 @@ gst_flacenc_chain (GstPad *pad, GstData *_data)
   {
     FLAC__SeekableStreamEncoderState state;
 
-    gst_flacenc_set_metadata (flacenc, flacenc->metadata);
+    FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
+                                          gst_flacenc_write_callback);
+    FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
+                                          gst_flacenc_seek_callback);
+                                          
+    FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
+                                          flacenc);
+		  
+    gst_flacenc_set_metadata (flacenc);
     state = FLAC__seekable_stream_encoder_init (flacenc->encoder);
     if (state != FLAC__STREAM_ENCODER_OK) {
       gst_element_error (GST_ELEMENT (flacenc),
@@ -728,8 +781,11 @@ gst_flacenc_change_state (GstElement *element)
       flacenc->first_buf = NULL;
       g_free (flacenc->data);
       flacenc->data = NULL;
-      FLAC__metadata_object_delete (flacenc->meta[0]);
-      g_free (flacenc->meta);
+      if (flacenc->meta) {
+	FLAC__metadata_object_delete (flacenc->meta[0]);
+	g_free (flacenc->meta);
+	flacenc->meta = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
     default:
