@@ -3,7 +3,7 @@
  *                    2000 Wim Taymans <wtay@chello.be>
  *                    2001 Bastien Nocera <hadess@hadess.net>
  *                    2002 Kristian Rietveld <kris@gtk.org>
- *                    2002 Colin Walters <walters@gnu.org>
+ *                    2002,2003 Colin Walters <walters@gnu.org>
  *
  * gnomevfssrc.c:
  *
@@ -26,6 +26,12 @@
 #define BROKEN_SIG 1
 /*#undef BROKEN_SIG */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "gstgnomevfs.h"
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -44,7 +50,6 @@
 /* gnome-vfs.h doesn't include the following header, which we need: */
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
-GstElementDetails gst_gnomevfssrc_details;
 
 #define GST_TYPE_GNOMEVFSSRC \
   (gst_gnomevfssrc_get_type())
@@ -77,8 +82,6 @@ struct _GstGnomeVFSSrc {
 
 	/* filename */
 	gchar *filename;
-	/* is it a local file ? */
-	gboolean is_local;
 	/* uri */
 	GnomeVFSURI *uri;
 
@@ -87,22 +90,17 @@ struct _GstGnomeVFSSrc {
 	/* Seek stuff */
 	gboolean need_flush;
 
-	/* local filename */
-	gchar *local_name;
-	/* fd for local file fallback */
-	gint fd;
-	/* mmap */
-	guchar *map;			/* where the file is mapped to */
-
 	/* details for fallback synchronous read */
 	GnomeVFSFileSize size;
 	GnomeVFSFileOffset curoffset;	/* current offset in file */
 	gulong bytes_per_read;		/* bytes per read */
 	gboolean new_seek;
+	gboolean in_first_get;
 
 	/* icecast/audiocast metadata extraction handling */
 	gboolean iradio_mode;
-	gboolean iradio_callbacks_pushed;
+	gboolean http_callbacks_pushed;
+	gboolean seekable;
 
 	gint icy_metaint;
 	GnomeVFSFileSize icy_count;
@@ -126,16 +124,12 @@ struct _GstGnomeVFSSrcClass {
 	GstElementClass parent_class;
 };
 
-/* elementfactory information */
-GstElementDetails gst_gnomevfssrc_details = {
+static GstElementDetails gst_gnomevfssrc_details = GST_ELEMENT_DETAILS (
 	"GnomeVFS Source",
 	"Source/File",
-	"LGPL",
 	"Read from any GnomeVFS file",
-	VERSION,
-	"Bastien Nocera <hadess@hadess.net>",
-	"(C) 2001",
-};
+	"Bastien Nocera <hadess@hadess.net>"
+);
 
 GST_PAD_FORMATS_FUNCTION (gst_gnomevfssrc_get_formats,
 	GST_FORMAT_BYTES
@@ -169,10 +163,10 @@ enum {
 	ARG_IRADIO_GENRE,
 	ARG_IRADIO_URL,
 	ARG_IRADIO_TITLE,
+	ARG_SEEKABLE,
 };
 
-GType gst_gnomevfssrc_get_type(void);
-
+static void 		gst_gnomevfssrc_base_init	(gpointer g_class);
 static void 		gst_gnomevfssrc_class_init	(GstGnomeVFSSrcClass *klass);
 static void 		gst_gnomevfssrc_init		(GstGnomeVFSSrc *gnomevfssrc);
 static void 		gst_gnomevfssrc_dispose		(GObject *object);
@@ -182,7 +176,7 @@ static void 		gst_gnomevfssrc_set_property	(GObject *object, guint prop_id,
 static void 		gst_gnomevfssrc_get_property	(GObject *object, guint prop_id,
 							 GValue *value, GParamSpec *pspec);
 
-static GstBuffer*	gst_gnomevfssrc_get		(GstPad *pad);
+static GstData*	gst_gnomevfssrc_get		(GstPad *pad);
 
 static GstElementStateReturn 
 			gst_gnomevfssrc_change_state	(GstElement *element);
@@ -207,7 +201,8 @@ GType gst_gnomevfssrc_get_type(void)
 
 	if (!gnomevfssrc_type) {
 		static const GTypeInfo gnomevfssrc_info = {
-			sizeof(GstGnomeVFSSrcClass),      NULL,
+			sizeof(GstGnomeVFSSrcClass), 
+			gst_gnomevfssrc_base_init,
 			NULL,
 			(GClassInitFunc) gst_gnomevfssrc_class_init,
 			NULL,
@@ -225,6 +220,14 @@ GType gst_gnomevfssrc_get_type(void)
 	return gnomevfssrc_type;
 }
 
+static void
+gst_gnomevfssrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &gst_gnomevfssrc_details);
+}
+
 static void gst_gnomevfssrc_class_init(GstGnomeVFSSrcClass *klass)
 {
 	GObjectClass *gobject_class;
@@ -280,6 +283,13 @@ static void gst_gnomevfssrc_class_init(GstGnomeVFSSrcClass *klass)
 							      "Name of currently playing song",
 							      NULL,
 							      G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class,
+					 ARG_SEEKABLE,
+					 g_param_spec_boolean ("seekable",
+							       "seekable",
+							       "TRUE is stream is seekable",
+							       FALSE,
+							       G_PARAM_READABLE));
 
 	gstelement_class->set_property = gst_gnomevfssrc_set_property;
 	gstelement_class->get_property = gst_gnomevfssrc_get_property;
@@ -303,19 +313,19 @@ static void gst_gnomevfssrc_init(GstGnomeVFSSrc *gnomevfssrc)
 			gst_gnomevfssrc_get_formats);
 	gst_element_add_pad(GST_ELEMENT(gnomevfssrc), gnomevfssrc->srcpad);
 
-	gnomevfssrc->filename = NULL;
-	gnomevfssrc->is_local = FALSE;
 	gnomevfssrc->uri = NULL;
 	gnomevfssrc->handle = NULL;
-	gnomevfssrc->fd = 0;
 	gnomevfssrc->curoffset = 0;
 	gnomevfssrc->bytes_per_read = 4096;
 	gnomevfssrc->new_seek = FALSE;
+	gnomevfssrc->in_first_get = TRUE;
 
 	gnomevfssrc->icy_metaint = 0;
 
+	gnomevfssrc->seekable = FALSE;
+
 	gnomevfssrc->iradio_mode = FALSE;
-	gnomevfssrc->iradio_callbacks_pushed = FALSE;
+	gnomevfssrc->http_callbacks_pushed = FALSE;
 	gnomevfssrc->icy_count = 0;
 	gnomevfssrc->iradio_name = NULL;
 	gnomevfssrc->iradio_genre = NULL;
@@ -374,7 +384,7 @@ static void gst_gnomevfssrc_set_property(GObject *object, guint prop_id, const G
 
 		g_free(src->filename);
 
-		/* clear the filename if we get a NULL (is that possible?) */
+		/* clear the filename if we get a NULL */
 		if (g_value_get_string (value) == NULL) {
 			gst_element_set_state(GST_ELEMENT(object), GST_STATE_NULL);
 			src->filename = NULL;
@@ -447,12 +457,48 @@ static void gst_gnomevfssrc_get_property(GObject *object, guint prop_id, GValue
 		g_value_set_string (value, src->iradio_title);
 		g_mutex_unlock(src->audiocast_udpdata_mutex);
 		break;
+	case ARG_SEEKABLE:
+		g_value_set_boolean (value, src->seekable);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+static char *
+unicodify (const char *str, int len, ...)
+{
+	char *ret = NULL, *cset;
+	va_list args;
+	gsize bytes_read, bytes_written;
+
+	if (g_utf8_validate (str, len, NULL))
+		return g_strndup (str, len >= 0 ? len : strlen (str));
+
+	va_start (args, len);
+	while ((cset = va_arg (args, char *)) != NULL)
+	{
+		if (!strcmp (cset, "locale"))
+			ret = g_locale_to_utf8 (str, len, &bytes_read,
+						&bytes_written, NULL);
+		else
+			ret = g_convert (str, len, "UTF-8", cset,
+					 &bytes_read, &bytes_written, NULL);
+		if (ret)
+			break;
+	}
+	va_end (args);
+
+	return ret;
+}
+
+static char *
+gst_gnomevfssrc_unicodify (const char *str)
+{
+	return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+}
+
 /*
  * icecast/audiocast metadata extraction support code
  */
@@ -463,17 +509,21 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	GError *error = NULL;
 	if (!src->iradio_mode)
 		return TRUE;
-	GST_DEBUG (0,"audiocast: registering listener");
+	GST_DEBUG ("audiocast: registering listener");
 	if (audiocast_register_listener(&src->audiocast_port, &src->audiocast_fd) < 0)
 	{
+		char *escaped;
+
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
 		gst_element_error(GST_ELEMENT(src),
 				  "opening vfs file \"%s\" (%s)",
-				  src->filename, 
+				  escaped,
 				  "unable to register UDP port");
+		g_free (escaped);
 		close(src->audiocast_fd);
 		return FALSE;
 	}
-	GST_DEBUG (0,"audiocast: creating pipe");
+	GST_DEBUG ("audiocast: creating pipe");
 	src->audiocast_notify_queue = NULL;
 	if (pipe(pipefds) < 0)
 	{
@@ -482,13 +532,16 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	}
 	src->audiocast_thread_die_infd = pipefds[0];
 	src->audiocast_thread_die_outfd = pipefds[1];
-	GST_DEBUG (0,"audiocast: creating audiocast thread");
+	GST_DEBUG ("audiocast: creating audiocast thread");
 	src->audiocast_thread = g_thread_create((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
 	if (error != NULL) {
+		char *escaped;
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
 		gst_element_error(GST_ELEMENT(src),
 				  "opening vfs file \"%s\" (unable to create thread: %s)",
-				  src->filename, 
+				  escaped,
 				  error->message);
+		g_free (escaped);
 		close(src->audiocast_fd);
 		return FALSE;
 	}
@@ -501,7 +554,7 @@ static int audiocast_register_listener(gint *port, gint *fd)
 	int sock;
 	socklen_t sinlen = sizeof (struct sockaddr_in);
 
-	GST_DEBUG (0,"audiocast: estabilishing UDP listener");
+	GST_DEBUG ("audiocast: estabilishing UDP listener");
 
 	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 		goto lose;
@@ -517,7 +570,7 @@ static int audiocast_register_listener(gint *port, gint *fd)
 	if (getsockname(sock, (struct sockaddr *)&sin, &sinlen) < 0)
 		goto lose_and_close;
 
-	GST_DEBUG (0,"audiocast: listening on local %s:%d", inet_ntoa(sin.sin_addr), g_ntohs(sin.sin_port));
+	GST_DEBUG ("audiocast: listening on local %s:%d", inet_ntoa(sin.sin_addr), g_ntohs(sin.sin_port));
 
 	*port = g_ntohs(sin.sin_port);
 	*fd = sock;
@@ -547,7 +600,6 @@ static void audiocast_do_notifications(GstGnomeVFSSrc *src)
 static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 {
 	char buf[1025], **lines;
-	char *valptr;
 	gsize len;
 	fd_set fdset, readset;
 	struct sockaddr_in from;
@@ -560,7 +612,7 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 
 	while (1)
 	{
-		GST_DEBUG (0,"audiocast thread: dropping into select");
+		GST_DEBUG ("audiocast thread: dropping into select");
 		readset = fdset;
 		if (select(FD_SETSIZE, &readset, NULL, NULL, NULL) < 0) {
 			perror("select");
@@ -568,19 +620,20 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 		}
 		if (FD_ISSET(src->audiocast_thread_die_infd, &readset)) {
 			char buf[1];
-			GST_DEBUG (0,"audiocast thread: got die character");
+			GST_DEBUG ("audiocast thread: got die character");
 			read(src->audiocast_thread_die_infd, buf, 1);
 			close(src->audiocast_thread_die_infd);
 			close(src->audiocast_fd);
 			return NULL;
 		}
-		GST_DEBUG (0,"audiocast thread: reading data");
+		GST_DEBUG ("audiocast thread: reading data");
 		len = recvfrom(src->audiocast_fd, buf, sizeof(buf) - 1, 0, (struct sockaddr *) &from, &fromlen);
 		if (len < 0 && errno == EAGAIN)
 			continue;
 		else if (len >= 0)
 		{
 			int i;
+			char *valptr, *value;
 			buf[len] = '\0';
 			lines = g_strsplit(buf, "\n", 0);
 			if (!lines)
@@ -601,35 +654,45 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 				g_strstrip(valptr);
 				if (!strlen(valptr))
 					continue;
+
+				value = gst_gnomevfssrc_unicodify (valptr);
+				if (!value)
+				{
+					g_print ("Unable to convert \"%s\" to UTF-8!\n", valptr);
+					continue;
+				}
 		
 				if (!strncmp(lines[i], "x-audiocast-streamtitle", 23)) {
 					g_mutex_lock(src->audiocast_udpdata_mutex);
-					if (src->iradio_title)
-						g_free(src->iradio_title);
-					src->iradio_title = g_strdup(valptr);
+					g_free(src->iradio_title);
+					src->iradio_title = value;
 					g_mutex_unlock(src->audiocast_udpdata_mutex);
+
 					g_mutex_lock(src->audiocast_queue_mutex);
 					src->audiocast_notify_queue = g_list_append(src->audiocast_notify_queue, "iradio-title");
-					GST_DEBUG(0,"audiocast title: %s\n", src->iradio_title);
+					GST_DEBUG ("audiocast title: %s\n", src->iradio_title);
 					g_mutex_unlock(src->audiocast_queue_mutex);
 				} else if (!strncmp(lines[i], "x-audiocast-streamurl", 21)) {
 					g_mutex_lock(src->audiocast_udpdata_mutex);
-					if (src->iradio_url)
-						g_free(src->iradio_url);
-					src->iradio_url = g_strdup(valptr);
+					g_free(src->iradio_url);
+					src->iradio_url = value;
 					g_mutex_unlock(src->audiocast_udpdata_mutex);
+
 					g_mutex_lock(src->audiocast_queue_mutex);
 					src->audiocast_notify_queue = g_list_append(src->audiocast_notify_queue, "iradio-url");
-					GST_DEBUG(0,"audiocast url: %s\n", src->iradio_title);
+					GST_DEBUG ("audiocast url: %s\n", src->iradio_title);
 					g_mutex_unlock(src->audiocast_queue_mutex);
 				} else if (!strncmp(lines[i], "x-audiocast-udpseqnr", 20)) {
 					gchar outbuf[120];
-					sprintf(outbuf, "x-audiocast-ack: %ld \r\n", atol(valptr));
+
+					sprintf(outbuf, "x-audiocast-ack: %ld \r\n", atol(value));
+					g_free (value);
+
 					if (sendto(src->audiocast_fd, outbuf, strlen(outbuf), 0, (struct sockaddr *) &from, fromlen) <= 0) {
-						fprintf(stderr, "Error sending response to server: %s\n", strerror(errno));
+						g_print("Error sending response to server: %s\n", strerror(errno));
 						continue;
 					}
-					GST_DEBUG(0,"sent audiocast ack: %s\n", outbuf);
+					GST_DEBUG ("sent audiocast ack: %s\n", outbuf);
 				}
 			}
 			g_strfreev(lines);
@@ -649,10 +712,10 @@ static void audiocast_thread_kill(GstGnomeVFSSrc *src)
 	  metadata, then we don't need the
 	  audiocast metadata too.
 	*/
-	GST_DEBUG (0,"audiocast: writing die character");
+	GST_DEBUG ("audiocast: writing die character");
 	write(src->audiocast_thread_die_outfd, "q", 1);
 	close(src->audiocast_thread_die_outfd);
-	GST_DEBUG (0,"audiocast: joining thread");
+	GST_DEBUG ("audiocast: joining thread");
 	g_thread_join(src->audiocast_thread);
 	src->audiocast_thread = NULL;
 }
@@ -670,7 +733,7 @@ gst_gnomevfssrc_send_additional_headers_callback (gconstpointer in,
 
 	if (!src->iradio_mode)
 		return;
-	GST_DEBUG(0,"sending headers\n");
+	GST_DEBUG ("sending headers\n");
 
         out_args->headers = g_list_append (out_args->headers,
                                            g_strdup ("icy-metadata:1\r\n"));
@@ -691,12 +754,13 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
         GnomeVFSModuleCallbackReceivedHeadersIn *in_args =
                 (GnomeVFSModuleCallbackReceivedHeadersIn *)in;
 
+	/* This is only used for internet radio stuff right now */
 	if (!src->iradio_mode)
 		return;
 
         for (i = in_args->headers; i; i = i->next) {
 		char *data = (char *) i->data;
-		char *key;
+		char *key = data;
 		char *value = strchr(data, ':');
 		if (!value)
 			continue;
@@ -711,7 +775,8 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 		{ 
                         sscanf (data + 12, "%d", &icy_metaint);
 			src->icy_metaint = icy_metaint;
-			GST_DEBUG (0,"got icy-metaint, killing audiocast thread");
+			GST_DEBUG ("got icy-metaint %d, killing audiocast thread",
+				   src->icy_metaint);
 			audiocast_thread_kill(src);
 			continue;
 		}
@@ -723,22 +788,22 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 		else
 			continue;
 
-		GST_DEBUG (0,"key: %s", key);
+		GST_DEBUG ("key: %s", key);
                 if (!strncmp (key, "name", 4)) {
+			g_free (src->iradio_name);
+			src->iradio_name = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_name)
-				g_free (src->iradio_name);
-			src->iradio_name = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-name");
+				g_object_notify (G_OBJECT (src), "iradio-name");
                 } else if (!strncmp (key, "genre", 5)) {
+			g_free (src->iradio_genre);
+			src->iradio_genre = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_genre)
-				g_free (src->iradio_genre);
-			src->iradio_genre = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-genre");
+				g_object_notify (G_OBJECT (src), "iradio-genre");
                 } else if (!strncmp (key, "url", 3)) {
+			g_free (src->iradio_url);
+			src->iradio_url = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_url)
-				g_free (src->iradio_url);
-			src->iradio_url = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-url");
+				g_object_notify (G_OBJECT (src), "iradio-url");
 		}
         }
 }
@@ -746,10 +811,10 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 static void
 gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc *gnomevfssrc)
 {
-	if (gnomevfssrc->iradio_callbacks_pushed)
+	if (gnomevfssrc->http_callbacks_pushed)
 		return;
 
-	GST_DEBUG (0,"pushing callbacks");
+	GST_DEBUG ("pushing callbacks");
 	gnome_vfs_module_callback_push (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS,
 					gst_gnomevfssrc_send_additional_headers_callback,
 					gnomevfssrc,
@@ -759,16 +824,16 @@ gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc *gnomevfssrc)
 					gnomevfssrc,
 					NULL);
 
-	gnomevfssrc->iradio_callbacks_pushed = TRUE;
+	gnomevfssrc->http_callbacks_pushed = TRUE;
 }
 
 static void
 gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc *gnomevfssrc)
 {
-	if (!gnomevfssrc->iradio_callbacks_pushed)
+	if (!gnomevfssrc->http_callbacks_pushed)
 		return;
 
-	GST_DEBUG (0,"popping callbacks");
+	GST_DEBUG ("popping callbacks");
 	gnome_vfs_module_callback_pop (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
 	gnome_vfs_module_callback_pop (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS);
 }
@@ -785,7 +850,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
 	gchar **tags;
 	int i;
 
-	GST_DEBUG (0,"reading icecast metadata");
+	GST_DEBUG ("reading icecast metadata");
 
         while (length == 0) {
                 res = gnome_vfs_read (src->handle, &foobyte, 1, &length);
@@ -819,21 +884,32 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
 	
 	for (i = 0; tags[i]; i++)
 	{
-		if (!g_strncasecmp(tags[i], "StreamTitle=", 12))
+		if (!g_ascii_strncasecmp(tags[i], "StreamTitle=", 12))
 		{
+			g_free (src->iradio_title);
+			src->iradio_title = gst_gnomevfssrc_unicodify (tags[i]+13);
 			if (src->iradio_title)
-				g_free (src->iradio_title);
-			src->iradio_title = g_strdup(tags[i]+13);
-			GST_DEBUG(0, "sending notification on icecast title");
-			g_object_notify (G_OBJECT (src), "iradio-title");
+			{
+				GST_DEBUG ("sending notification on icecast title");
+				g_object_notify (G_OBJECT (src), "iradio-title");
+			}
+			else
+				g_print ("Unable to convert icecast title \"%s\" to UTF-8!\n",
+					 tags[i]+13);
+			
 		}
-		if (!g_strncasecmp(tags[i], "StreamUrl=", 10))
+		if (!g_ascii_strncasecmp(tags[i], "StreamUrl=", 10))
 		{
+			g_free (src->iradio_url);
+			src->iradio_url = gst_gnomevfssrc_unicodify (tags[i]+11);
 			if (src->iradio_url)
-				g_free (src->iradio_url);
-			src->iradio_url = g_strdup(tags[i]+11);
-			GST_DEBUG(0, "sending notification on icecast url");
-			g_object_notify (G_OBJECT (src), "iradio-url");
+			{
+				GST_DEBUG ("sending notification on icecast url");
+				g_object_notify (G_OBJECT (src), "iradio-url");
+			}
+			else
+				g_print ("Unable to convert icecast url \"%s\" to UTF-8!\n",
+					 tags[i]+11);
 		}
 	}
 
@@ -848,12 +924,13 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
  *
  * Push a new buffer from the gnomevfssrc at the current offset.
  */
-static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
+static GstData *gst_gnomevfssrc_get(GstPad *pad)
 {
 	GstGnomeVFSSrc *src;
 	GnomeVFSResult result = 0;
 	GstBuffer *buf;
 	GnomeVFSFileSize readbytes;
+	guint8 *data;
 
 	g_return_val_if_fail(pad != NULL, NULL);
 	src = GST_GNOMEVFSSRC(gst_pad_get_parent(pad));
@@ -864,7 +941,7 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 	if ((src->curoffset >= src->size) && (src->size != 0))
 	{
 		gst_element_set_eos (GST_ELEMENT (src));
-		return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+		return GST_DATA (gst_event_new (GST_EVENT_EOS));
 	}
 
 	/* create the buffer */
@@ -874,65 +951,43 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 
 	audiocast_do_notifications(src);
 
-	/* read it in from the file */
-	if (src->is_local)
-	{
-		/* simply set the buffer to point to the correct region of the
-		 * file */
-		GST_BUFFER_DATA (buf) = src->map + src->curoffset;
-		GST_BUFFER_OFFSET (buf) = src->curoffset;
-		GST_BUFFER_FLAG_SET (buf, GST_BUFFER_DONTFREE);
-		GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY);
-
-		if ((src->curoffset + src->bytes_per_read) > src->size)
-		{
-			GST_BUFFER_SIZE (buf) = src->size - src->curoffset;
-		} else {
-			GST_BUFFER_SIZE (buf) = src->bytes_per_read;
-		}
-
-		if (src->new_seek)
-		{
-			GstEvent *event;
-
-			gst_buffer_unref (buf);
-			GST_DEBUG (0,"new seek %lld", src->curoffset);
-			src->new_seek = FALSE;
-
-			GST_DEBUG (GST_CAT_EVENT, "gnomevfssrc sending discont");
-			event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
-			src->need_flush = FALSE;
-			return GST_BUFFER (event);
-		}
-
-		src->curoffset += GST_BUFFER_SIZE (buf);
-
-	} else if (src->iradio_mode && src->icy_metaint > 0) {
+	if (src->iradio_mode && src->icy_metaint > 0) {
 		GST_BUFFER_DATA (buf) = g_malloc0 (src->icy_metaint);
+		data = GST_BUFFER_DATA (buf);
 		g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 
-		result = gnome_vfs_read (src->handle, GST_BUFFER_DATA (buf),
-					 src->icy_metaint - src->icy_count,
-					 &readbytes);
-
-		/* EOS? */
-		if (readbytes == 0) {
-			gst_buffer_unref (buf);
-			gst_element_set_eos (GST_ELEMENT (src));
-
-			return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
-		}
-
-		src->icy_count += readbytes;
-		GST_BUFFER_OFFSET (buf) = src->curoffset;
-		GST_BUFFER_SIZE (buf) = readbytes;
-		src->curoffset += readbytes;
-
-		if (src->icy_count == src->icy_metaint) {
-			gst_gnomevfssrc_get_icy_metadata (src);
-			src->icy_count = 0;
-		}
+		GST_BUFFER_SIZE (buf) = 0;
+		/* FIXME GROSS HACK: We try to read in at least 8000
+		 * bytes of data so that mp3 typefinding will work. */
+		do
+		{
+			GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
+			result = gnome_vfs_read (src->handle, data,
+						 src->icy_metaint - src->icy_count,
+						 &readbytes);
 
+			/* EOS? */
+			if (readbytes == 0) {
+				gst_buffer_unref (buf);
+				gst_element_set_eos (GST_ELEMENT (src));
+				src->in_first_get = FALSE;
+				return GST_DATA (gst_event_new (GST_EVENT_EOS));
+			}
+			
+			src->icy_count += readbytes;
+			GST_BUFFER_OFFSET (buf) = src->curoffset;
+			GST_BUFFER_SIZE (buf) += readbytes;
+		      	data += readbytes;
+			src->curoffset += readbytes;
+			
+			if (src->icy_count == src->icy_metaint) {
+				gst_gnomevfssrc_get_icy_metadata (src);
+				src->icy_count = 0;
+			}
+		} while (src->in_first_get
+			 && GST_BUFFER_OFFSET (buf) < 8000 &&
+			 src->icy_metaint - src->icy_count >= 8000);
+		src->in_first_get = FALSE;
 	} else {
 		/* allocate the space for the buffer data */
 		GST_BUFFER_DATA(buf) = g_malloc(src->bytes_per_read);
@@ -943,20 +998,20 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 			GstEvent *event;
 
 			gst_buffer_unref (buf);
-			GST_DEBUG (0,"new seek %lld", src->curoffset);
+			GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
 			src->new_seek = FALSE;
 
-			GST_DEBUG (GST_CAT_EVENT, "gnomevfssrc sending discont");
+			GST_DEBUG ("gnomevfssrc sending discont");
 			event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
 			src->need_flush = FALSE;
 
-			return GST_BUFFER (event);
+			return GST_DATA (event);
 		}
 
 		result = gnome_vfs_read(src->handle, GST_BUFFER_DATA(buf),
 				   src->bytes_per_read, &readbytes);
 
-		GST_DEBUG(0, "read: %s, readbytes: %Lu",
+		GST_DEBUG ("read: %s, readbytes: %" G_GINT64_FORMAT,
 				gnome_vfs_result_to_string(result), readbytes);
 		/* deal with EOS */
 		if (readbytes == 0)
@@ -965,7 +1020,7 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 
 			gst_element_set_eos (GST_ELEMENT (src));
 
-			return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+			return GST_DATA (gst_event_new (GST_EVENT_EOS));
 		}
 		
 		GST_BUFFER_OFFSET(buf) = src->curoffset;
@@ -976,13 +1031,14 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 	GST_BUFFER_TIMESTAMP (buf) = -1;
 
 	/* we're done, return the buffer */
-	return buf;
+	return GST_DATA (buf);
 }
 
-/* open the file and mmap it, necessary to go to READY state */
+/* open the file, do stuff necessary to go to READY state */
 static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 {
 	GnomeVFSResult result;
+	GnomeVFSFileInfo *info;
 
 	g_return_val_if_fail(!GST_FLAG_IS_SET(src, GST_GNOMEVFSSRC_OPEN),
 			     FALSE);
@@ -990,85 +1046,60 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 	/* create the uri */
 	src->uri = gnome_vfs_uri_new(src->filename);
 	if (!src->uri) {
-		gst_element_error(GST_ELEMENT(src),
-				  "creating uri \"%s\" (%s)",
-				  	src->filename, strerror (errno));
+		gst_element_error(GST_ELEMENT(src), "creating uri \"%s\" (%s)",
+				  src->filename, strerror (errno));
 		return FALSE;
 	}
 
-	/* open the file using open() if the file is local */
-	src->is_local = gnome_vfs_uri_is_local(src->uri);
-
-	if (src->is_local) {
-		src->local_name =
-			gnome_vfs_get_local_path_from_uri(src->filename);
-		src->fd = open(src->local_name, O_RDONLY);
-		if (src->fd < 0)
-		{
-			gst_element_error(GST_ELEMENT(src),
-					  "opening local file \"%s\" (%s)",
-						src->filename, strerror (errno));
-			return FALSE;
-		}
-
-		/* find the file length */
-		src->size = lseek (src->fd, 0, SEEK_END);
-		lseek (src->fd, 0, SEEK_SET);
-		src->map = mmap (NULL, src->size, PROT_READ, MAP_SHARED,
-				src->fd, 0);
-		madvise (src->map,src->size, 2);
-		/* collapse state if that failed */
-		if (src->map == NULL)
-		{
-			gst_gnomevfssrc_close_file(src);
-			gst_element_error (GST_ELEMENT (src),"mmapping file");
-			return FALSE;
-		}
-
-		src->new_seek = TRUE;
-	} else {
-
-		if (!audiocast_init(src))
-			return FALSE;
+	if (!audiocast_init(src))
+		return FALSE;
 			
-		gst_gnomevfssrc_push_callbacks (src);
-
-		result =
-		    gnome_vfs_open_uri(&(src->handle), src->uri,
-				       GNOME_VFS_OPEN_READ);
-		if (result != GNOME_VFS_OK)
-		{
-			gst_gnomevfssrc_pop_callbacks (src);
-			audiocast_thread_kill(src);
-			gst_element_error(GST_ELEMENT(src),
-				  "opening vfs file \"%s\" (%s)",
-				      src->filename, 
-				      gnome_vfs_result_to_string(result));
-			return FALSE;
-		}
-
-		/* find the file length */
-		{
-			GnomeVFSResult size_result;
-			GnomeVFSFileInfo *info;
-
-			info = gnome_vfs_file_info_new ();
-			size_result = gnome_vfs_get_file_info_uri(src->uri,
-					info, GNOME_VFS_FILE_INFO_DEFAULT);
-
-			if (size_result != GNOME_VFS_OK)
-				src->size = 0;
-			else
-				src->size = info->size;
-
-		        GST_DEBUG(0, "size %lld", src->size);
-
-			gnome_vfs_file_info_unref(info);
-		}
-
-		audiocast_do_notifications(src);
+	gst_gnomevfssrc_push_callbacks (src);
 	
-		GST_DEBUG(0, "open %s", gnome_vfs_result_to_string(result));
+	result = gnome_vfs_open_uri(&(src->handle), src->uri,
+				    GNOME_VFS_OPEN_READ);
+	if (result != GNOME_VFS_OK)
+	{
+		char *escaped;
+		
+		gst_gnomevfssrc_pop_callbacks (src);
+		audiocast_thread_kill(src);
+
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
+		gst_element_error(GST_ELEMENT(src),
+				  "opening vfs file \"%s\" (%s)",
+				  escaped,
+				  gnome_vfs_result_to_string(result));
+		g_free (escaped);
+		return FALSE;
+	}
+	
+	info = gnome_vfs_file_info_new ();
+	if (gnome_vfs_get_file_info_from_handle (src->handle, info,
+						 GNOME_VFS_FILE_INFO_DEFAULT)
+	    == GNOME_VFS_OK)
+	{
+		if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
+			src->size = info->size;
+	}
+	else
+		GST_DEBUG ("getting info failed: %s", gnome_vfs_result_to_string (result));
+
+	gnome_vfs_file_info_unref(info);
+	
+	GST_DEBUG ("size %" G_GINT64_FORMAT, src->size);
+	
+	audiocast_do_notifications(src);
+	
+	GST_DEBUG ("open result: %s", gnome_vfs_result_to_string (result));
+
+	src->in_first_get = TRUE;
+
+	if(gnome_vfs_seek(src->handle, GNOME_VFS_SEEK_CURRENT, 0)
+	    == GNOME_VFS_OK){
+		src->seekable = TRUE;
+	}else{
+		src->seekable = FALSE;
 	}
 
 	GST_FLAG_SET(src, GST_GNOMEVFSSRC_OPEN);
@@ -1076,7 +1107,6 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 	return TRUE;
 }
 
-/* unmap and close the file */
 static void gst_gnomevfssrc_close_file(GstGnomeVFSSrc *src)
 {
 	g_return_if_fail(GST_FLAG_IS_SET(src, GST_GNOMEVFSSRC_OPEN));
@@ -1084,22 +1114,9 @@ static void gst_gnomevfssrc_close_file(GstGnomeVFSSrc *src)
 	gst_gnomevfssrc_pop_callbacks (src);
 	audiocast_thread_kill(src);
 
-	/* close the file */
-	if (src->is_local)
-	{
-		/* unmap the file from memory */
-		munmap (src->map, src->size);
-		close(src->fd);
-	} else {
-		gnome_vfs_close(src->handle);
-	}
+	gnome_vfs_close(src->handle);
 
-	/* zero out a lot of our state */
-	src->is_local = FALSE;
 	gnome_vfs_uri_unref(src->uri);
-	g_free(src->local_name);
-	src->fd = 0;
-	src->map = NULL;
 	src->size = 0;
 	src->curoffset = 0;
 	src->new_seek = FALSE;
@@ -1138,22 +1155,6 @@ static GstElementStateReturn gst_gnomevfssrc_change_state(GstElement *element)
 	return GST_STATE_SUCCESS;
 }
 
-static gboolean plugin_init(GModule *module, GstPlugin *plugin)
-{
-	GstElementFactory *factory;
-
-	/* create an elementfactory for the aasink element */
-	factory =
-	    gst_element_factory_new("gnomevfssrc", GST_TYPE_GNOMEVFSSRC,
-				   &gst_gnomevfssrc_details);
-	g_return_val_if_fail(factory != NULL, FALSE);
-
-	gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-	return TRUE;
-}
-
-
 static gboolean
 gst_gnomevfssrc_srcpad_query (GstPad *pad, GstQueryType type,
 		              GstFormat *format, gint64 *value)
@@ -1168,10 +1169,18 @@ gst_gnomevfssrc_srcpad_query (GstPad *pad, GstQueryType type,
 		*value = src->size;
 		break;
 	case GST_QUERY_POSITION:
-		if (*format != GST_FORMAT_BYTES) {
-			return FALSE;
+		switch (*format) {
+			case GST_FORMAT_BYTES:
+				*value = src->curoffset;
+		                break;
+			case GST_FORMAT_PERCENT:
+				if (src->size == 0)
+					return FALSE;
+				*value = src->curoffset * GST_FORMAT_PERCENT_MAX / src->size;
+				break;
+			default:
+				return FALSE;
 		}
-		*value = src->curoffset;
 		break;
 	default:
 		return FALSE;
@@ -1189,6 +1198,7 @@ gst_gnomevfssrc_srcpad_event (GstPad *pad, GstEvent *event)
 	case GST_EVENT_SEEK:
         {
 		gint64 desired_offset;
+		GnomeVFSResult result;
 
 		if (GST_EVENT_SEEK_FORMAT (event) != GST_FORMAT_BYTES) {
 			gst_event_unref (event);
@@ -1209,18 +1219,15 @@ gst_gnomevfssrc_srcpad_event (GstPad *pad, GstEvent *event)
 			return FALSE;
 			break;
 		}
-		if (!src->is_local) {
-			GnomeVFSResult result;
-
-			result = gnome_vfs_seek(src->handle,
+		
+		result = gnome_vfs_seek(src->handle,
 					GNOME_VFS_SEEK_START, desired_offset);
-			GST_DEBUG(0, "new_seek: %s",
-					gnome_vfs_result_to_string(result));
-			
-			if (result != GNOME_VFS_OK) {
-				gst_event_unref (event);
-				return FALSE;
-			}
+		GST_DEBUG ("new_seek: %s",
+			  gnome_vfs_result_to_string(result));
+		
+		if (result != GNOME_VFS_OK) {
+			gst_event_unref (event);
+			return FALSE;
 		}
 		src->curoffset = desired_offset;
 		src->new_seek = TRUE;
@@ -1248,16 +1255,3 @@ gst_gnomevfssrc_srcpad_event (GstPad *pad, GstEvent *event)
 
 	return TRUE;
 }
-
-GstPluginDesc plugin_desc = {
-	GST_VERSION_MAJOR,
-	GST_VERSION_MINOR,
-	"gnomevfssrc",
-	plugin_init
-};
-
-/*
-  Local Variables:
-  c-file-style: "linux"
-  End:
-*/
