@@ -17,10 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <fame.h>
 #include <string.h>
+#include <math.h>
 
 #include "gstlibfame.h"
+#include <gst/video/video.h>
 
 #define FAMEENC_BUFFER_SIZE (300 * 1024) 
 
@@ -29,10 +34,7 @@ static GstElementDetails gst_fameenc_details = {
   "MPEG1 and MPEG4 video encoder using the libfame library",
   "Codec/Video/Encoder",
   "Uses fameenc to encode MPEG video streams",
-  VERSION,
-  "fameenc: (C) 2000-2001, Vivien Chappelier\n"
-  "Thomas Vander Stichele <thomas@apestaart.org>",
-  "(C) 2002",
+  "Thomas Vander Stichele <thomas@apestaart.org>"
 };
 
 static GQuark fame_object_name;
@@ -47,7 +49,6 @@ enum {
 enum {
   ARG_0,
   ARG_VERSION,
-  ARG_FRAMERATE,
   ARG_BITRATE,
   ARG_QUALITY,
   ARG_PATTERN,
@@ -65,10 +66,21 @@ GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "fameenc_sink_caps",
-    "video/raw",
-      "format",		GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0')),
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096)
+    "video/x-raw-yuv",
+      "format",
+	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0')),
+      "width",        GST_PROPS_INT_RANGE (16, 4096),
+      "height",       GST_PROPS_INT_RANGE (16, 4096),
+      "framerate",    GST_PROPS_LIST (
+			GST_PROPS_FLOAT (24/1.001),
+			GST_PROPS_FLOAT (24.),
+			GST_PROPS_FLOAT (25.),
+			GST_PROPS_FLOAT (30/1.001),
+			GST_PROPS_FLOAT (30.),
+			GST_PROPS_FLOAT (50.),
+			GST_PROPS_FLOAT (60/1.001),
+			GST_PROPS_FLOAT (60.)
+		      )
   )
 )
 
@@ -79,13 +91,27 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   GST_CAPS_NEW (
     "fameenc_src_caps",
     "video/mpeg",
-      "mpegversion", GST_PROPS_LIST (
-	  GST_PROPS_INT (1), GST_PROPS_INT (4)),
-      "systemstream", GST_PROPS_BOOLEAN (FALSE)
+      "mpegversion",  GST_PROPS_LIST (
+			GST_PROPS_INT (1),
+			GST_PROPS_INT (4)
+		      ),
+      "systemstream", GST_PROPS_BOOLEAN (FALSE),
+      "width",        GST_PROPS_INT_RANGE (16, 4096),
+      "height",       GST_PROPS_INT_RANGE (16, 4096),
+      "framerate",    GST_PROPS_LIST (
+			GST_PROPS_FLOAT (24/1.001),
+			GST_PROPS_FLOAT (24.),
+			GST_PROPS_FLOAT (25.),
+			GST_PROPS_FLOAT (30/1.001),
+			GST_PROPS_FLOAT (30.),
+			GST_PROPS_FLOAT (50.),
+			GST_PROPS_FLOAT (60/1.001),
+			GST_PROPS_FLOAT (60.)
+		      )
   )
 );
 
-#define MAX_FRAME_RATES  16
+#define MAX_FRAME_RATES  9
 typedef struct
 {
   gint num;
@@ -103,49 +129,34 @@ static const frame_rate_entry frame_rates[] =
   { 50, 1 },
   { 60000, 1001 },
   { 60, 1 },
-  { 0, 0 },
-  { 0, 0 },
-  { 0, 0 },
-  { 0, 0 },
-  { 0, 0 },
-  { 0, 0 },
-  { 0, 0 },
 };
 
 static gint
-framerate_to_index (num, den)
+framerate_to_index (gfloat fps)
 {
   gint i;
+  gint idx = -1;
   
-  for (i = 0; i < MAX_FRAME_RATES; i++) {
-    if (frame_rates[i].num == num && frame_rates[i].den == den)
-      return i;
-  }
-  return 0;
-}
+  for (i = 1; i < MAX_FRAME_RATES; i++) {
+    if (idx == -1) {
+      idx = i;
+    } else {
+      gfloat old_diff = fabs((1. * frame_rates[idx].num /
+				frame_rates[idx].den) - fps),
+             new_diff = fabs((1. * frame_rates[i].num /
+				frame_rates[i].den) - fps);
 
-#define GST_TYPE_FAMEENC_FRAMERATE (gst_fameenc_framerate_get_type())
-static GType
-gst_fameenc_framerate_get_type(void) {
-  static GType fameenc_framerate_type = 0;
-  static GEnumValue fameenc_framerate[] = {
-    {1, "1", "24000/1001 (23.97)"},
-    {2, "2", "24"},
-    {3, "3", "25"},
-    {4, "4", "30000/1001 (29.97)"},
-    {5, "5", "30"},
-    {6, "6", "50"},
-    {7, "7", "60000/1001 (59.94)"},
-    {8, "8", "60"},
-    {0, NULL, NULL},
-  };
-  if (!fameenc_framerate_type) {
-    fameenc_framerate_type = g_enum_register_static("GstFameEncFrameRate", fameenc_framerate);
+      if (new_diff < old_diff) {
+        idx = i;
+      }
+    }
   }
-  return fameenc_framerate_type;
+
+  return idx;
 }
 
 static void	gst_fameenc_class_init		(GstFameEncClass *klass);
+static void	gst_fameenc_base_init		(GstFameEncClass *klass);
 static void	gst_fameenc_init		(GstFameEnc *fameenc);
 static void	gst_fameenc_dispose		(GObject *object);
 
@@ -154,7 +165,7 @@ static void	gst_fameenc_set_property	(GObject *object, guint prop_id,
 static void	gst_fameenc_get_property	(GObject *object, guint prop_id, 
 						 GValue *value, GParamSpec *pspec);
 
-static void	gst_fameenc_chain		(GstPad *pad, GstBuffer *buf);
+static void	gst_fameenc_chain		(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_fameenc_signals[LAST_SIGNAL] = { 0 };*/
@@ -167,7 +178,7 @@ gst_fameenc_get_type (void)
   if (!fameenc_type) {
     static const GTypeInfo fameenc_info = {
       sizeof (GstFameEncClass),      
-      NULL,
+      (GBaseInitFunc) gst_fameenc_base_init,
       NULL,
       (GClassInitFunc) gst_fameenc_class_init,
       NULL,
@@ -188,6 +199,19 @@ gst_fameenc_item_compare (fame_list_t *item1, fame_list_t *item2)
   return strcmp (item1->type, item2->type);
 }
 
+static void
+gst_fameenc_base_init (GstFameEncClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (src_template_factory));
+
+  gst_element_class_set_details (element_class, &gst_fameenc_details);
+}
+
 static void
 gst_fameenc_class_init (GstFameEncClass *klass)
 {
@@ -272,9 +296,6 @@ gst_fameenc_class_init (GstFameEncClass *klass)
     }
   }
 
-  g_object_class_install_property (gobject_class, ARG_FRAMERATE,
-    g_param_spec_enum ("framerate", "Frame Rate", "Number of frames per second",
-                       GST_TYPE_FAMEENC_FRAMERATE, 3, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_BITRATE,
     g_param_spec_int ("bitrate", "Bitrate", "Target bitrate (0 = VBR)",
                       0, 5000000, 0, G_PARAM_READWRITE));
@@ -299,7 +320,8 @@ gst_fameenc_class_init (GstFameEncClass *klass)
 static GstPadLinkReturn
 gst_fameenc_sinkconnect (GstPad *pad, GstCaps *caps)
 {
-  gint width, height;
+  gint width, height, fps_idx;
+  gfloat fps;
   GstFameEnc *fameenc;
 
   fameenc = GST_FAMEENC (gst_pad_get_parent (pad));
@@ -308,17 +330,23 @@ gst_fameenc_sinkconnect (GstPad *pad, GstCaps *caps)
     return GST_PAD_LINK_DELAYED;
 
   if (fameenc->initialized) {
-    GST_DEBUG(0, "error: fameenc encoder already initialized !");
+    GST_DEBUG ("error: fameenc encoder already initialized !");
     return GST_PAD_LINK_REFUSED;
   }
 
   gst_caps_get_int (caps, "width", &width);
   gst_caps_get_int (caps, "height", &height);
+  gst_caps_get_float (caps, "framerate", &fps);
   
   /* fameenc requires width and height to be multiples of 16 */
   if (width % 16 != 0 || height % 16 != 0)
     return GST_PAD_LINK_REFUSED;
 
+  fps_idx = framerate_to_index (fps);
+  fameenc->fp.frame_rate_num = frame_rates[fps_idx].num;
+  fameenc->fp.frame_rate_den = frame_rates[fps_idx].den;
+  fameenc->time_interval = 0;
+
   fameenc->fp.width = width;
   fameenc->fp.height = height;
   fameenc->fp.coding = (const char *) fameenc->pattern;
@@ -397,8 +425,9 @@ gst_fameenc_dispose (GObject *object)
 }
 
 static void
-gst_fameenc_chain (GstPad *pad, GstBuffer *buf)
+gst_fameenc_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstFameEnc *fameenc;
   guchar *data;
   gulong size;
@@ -415,7 +444,7 @@ gst_fameenc_chain (GstPad *pad, GstBuffer *buf)
   data = (guchar *) GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
 
-  GST_DEBUG (0,"gst_fameenc_chain: got buffer of %ld bytes in '%s'", 
+  GST_DEBUG ("gst_fameenc_chain: got buffer of %ld bytes in '%s'", 
 	     size, GST_OBJECT_NAME (fameenc));
 
   /* the data contains the three planes side by side, with size w * h, w * h /4,
@@ -442,7 +471,7 @@ gst_fameenc_chain (GstPad *pad, GstBuffer *buf)
       g_warning ("FAMEENC_BUFFER_SIZE is defined too low, encoded slice has size %d !\n", length);
 
     if (!fameenc->time_interval) {
-  	fameenc->time_interval = GST_SECOND / fameenc->fp.frame_rate_num;
+  	fameenc->time_interval = GST_SECOND * fameenc->fp.frame_rate_den / fameenc->fp.frame_rate_num;
     }
 
     fameenc->next_time += fameenc->time_interval;
@@ -453,10 +482,10 @@ gst_fameenc_chain (GstPad *pad, GstBuffer *buf)
     memcpy (GST_BUFFER_DATA(outbuf), fameenc->buffer, length);
     GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
 
-    GST_DEBUG (0,"gst_fameenc_chain: pushing buffer of size %d",
+    GST_DEBUG ("gst_fameenc_chain: pushing buffer of size %d",
                GST_BUFFER_SIZE(outbuf));
 
-    gst_pad_push (fameenc->srcpad, outbuf);
+    gst_pad_push (fameenc->srcpad, GST_DATA (outbuf));
   }
 
   fame_end_frame (fameenc->fc, NULL); 
@@ -474,20 +503,11 @@ gst_fameenc_set_property (GObject *object, guint prop_id,
   fameenc = GST_FAMEENC (object);
 
   if (fameenc->initialized) {
-    GST_DEBUG(0, "error: fameenc encoder already initialized, cannot set properties !");
+    GST_DEBUG ("error: fameenc encoder already initialized, cannot set properties !");
     return;
   }
 
   switch (prop_id) {
-    case ARG_FRAMERATE:
-    {
-      gint index = g_value_get_enum (value);
-
-      fameenc->fp.frame_rate_num = frame_rates[index].num;
-      fameenc->fp.frame_rate_den = frame_rates[index].den;
-      fameenc->time_interval = 0;
-      break;
-    }
     case ARG_BITRATE:
       fameenc->fp.bitrate = g_value_get_int (value);
       break;
@@ -534,13 +554,6 @@ gst_fameenc_get_property (GObject *object, guint prop_id,
   fameenc = GST_FAMEENC (object);
 
   switch (prop_id) {
-    case ARG_FRAMERATE:
-    {
-      gint index = framerate_to_index (fameenc->fp.frame_rate_num, 
-		                       fameenc->fp.frame_rate_den);
-      g_value_set_enum (value, index);
-      break;
-    }
     case ARG_BITRATE:
       g_value_set_int (value, fameenc->fp.bitrate);
       break;
@@ -585,28 +598,20 @@ gst_fameenc_get_property (GObject *object, guint prop_id,
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the fameenc element */
-  factory = gst_element_factory_new ("fameenc", GST_TYPE_FAMEENC,
-                                     &gst_fameenc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, 
-      GST_PAD_TEMPLATE_GET (sink_template_factory));
-  gst_element_factory_add_pad_template (factory, 
-      GST_PAD_TEMPLATE_GET (src_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "fameenc",
+			       GST_RANK_NONE, GST_TYPE_FAMEENC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "fameenc",
-  plugin_init
-};
+  "Fast Assembly MPEG Encoder",
+  plugin_init,
+  LIBFAME_VERSION,
+  "LGPL",
+  "libfame",
+  "http://fame.sourceforge.net/"
+)
