@@ -77,6 +77,7 @@ sink_template_factory (void)
         "icecastsend_sink",
         "audio/mpeg",
 	gst_props_new (
+	  "mpegversion", GST_PROPS_INT (1),
 	  "layer", GST_PROPS_INT_RANGE (1, 3),
 	  NULL
 	)),
@@ -109,7 +110,7 @@ gst_icecastsend_get_type (void)
   if (!icecastsend_type) {
     static const GTypeInfo icecastsend_info = {
       sizeof (GstIcecastSendClass),
-      (GBaseInitFunc gst_icecastsend_base_init, NULL,
+      (GBaseInitFunc) gst_icecastsend_base_init, NULL,
       (GClassInitFunc) gst_icecastsend_class_init, NULL, NULL,
       sizeof (GstIcecastSend), 0,
       (GInstanceInitFunc) gst_icecastsend_init,
@@ -473,7 +474,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "GPL",
-  GST_COPYRIGHT,
   "libshout",
   "http://developer.icecast.org/libshout/"
 )
