@@ -1341,6 +1341,9 @@ typedef struct AVCodecContext {
      * - decoding: unused
      */
     int lmax;
+
+    /* Added for ffmpeg gstreamer plugin (ffdec_wmav1 & ffdec_wmav2) */
+    int wma_flags1, wma_flags2;
     
 } AVCodecContext;
 
