@@ -44,7 +44,7 @@
 #include <gst/video/video.h>
 
 /* for audio codec IDs */
-#include <gst/riff/riff.h>
+#include <gst/riff/riff-ids.h>
 
 #include "gstasfmux.h"
 
@@ -183,6 +183,7 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
   GST_CAPS_NEW (
     "asfmux_sink_audio_mpeg",
     "audio/mpeg",
+      "mpegversion",      GST_PROPS_INT (1),
       "layer",		  GST_PROPS_INT_RANGE (1, 3),
       "rate",             GST_PROPS_INT_RANGE (1000, 96000),
       "channels",         GST_PROPS_INT_RANGE (1, 2)
@@ -1381,23 +1382,3 @@ gst_asfmux_change_state (GstElement *element)
   return GST_STATE_SUCCESS;
 }
 
-static gboolean
-plugin_init (GstPlugin *plugin)
-{
-  if (!gst_element_register (plugin, "asfmux", GST_RANK_NONE, GST_TYPE_ASFMUX))
-    return FALSE;
-  
-  return TRUE;
-}
-
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "asfmux",
-  "Muxes audio and video into an ASF stream",
-  plugin_init,
-  VERSION,
-  "LGPL",
-  GST_COPYRIGHT,
-  GST_PACKAGE,
-  GST_ORIGIN)
