@@ -97,6 +97,7 @@ gst_ximagesink_ximage_new (GstXImageSink *ximagesink, gint width, gint height)
       XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo);
     }
   else
+#endif /* HAVE_XSHM */
     {
       ximage->data = g_malloc (ximage->size);
       
@@ -108,17 +109,6 @@ gst_ximagesink_ximage_new (GstXImageSink *ximagesink, gint width, gint height)
                                      ximagesink->xcontext->bpp,
                                      ximage->width * (ximagesink->xcontext->bpp / 8));
     }
-#else
-  ximage->data = g_malloc (ximage->size);
-    
-  ximage->ximage = XCreateImage (ximagesink->xcontext->disp,
-                                 ximagesink->xcontext->visual, 
-                                 ximagesink->xcontext->depth,
-                                 ZPixmap, 0, ximage->data, 
-                                 ximage->width, ximage->height,
-                                 ximagesink->xcontext->bpp,
-                                 ximage->width * (ximagesink->xcontext->bpp / 8));
-#endif /* HAVE_XSHM */
   
   if (ximage->ximage)
     {
@@ -165,15 +155,12 @@ gst_ximagesink_ximage_destroy (GstXImageSink *ximagesink, GstXImage *ximage)
         shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
     }
   else
+#endif /* HAVE_XSHM */ 
     {
       if (ximage->ximage)
         XDestroyImage (ximage->ximage);
     }
-#else
-  if (ximage->ximage)
-    XDestroyImage (ximage->ximage);
-#endif /* HAVE_XSHM */ 
-  
+
   g_mutex_unlock (ximagesink->x_lock);
   
   g_free (ximage);
@@ -203,16 +190,12 @@ gst_ximagesink_ximage_put (GstXImageSink *ximagesink, GstXImage *ximage)
                     FALSE);
     }
   else
+#endif /* HAVE_XSHM */
     {
       XPutImage (ximagesink->xcontext->disp, ximagesink->xwindow->win, 
                  ximagesink->xwindow->gc, ximage->ximage,  
                  0, 0, x, y, ximage->width, ximage->height);
     }
-#else
-  XPutImage (ximagesink->xcontext->disp, ximagesink->xwindow->win, 
-             ximagesink->xwindow->gc, ximage->ximage,  
-             0, 0, x, y, ximage->width, ximage->height);
-#endif /* HAVE_XSHM */
   
   XSync(ximagesink->xcontext->disp, FALSE);
   
@@ -382,7 +365,7 @@ gst_ximagesink_handle_xevents (GstXImageSink *ximagesink, GstPad *pad)
                                        e.xkey.keycode, 0);
             gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
                                            XKeysymToString (keysym));
-            /* What's that ? */
+            /* FIXME : What's that ? */
             gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
                                            "unknown");
             break;
@@ -465,15 +448,16 @@ gst_ximagesink_xcontext_get (GstXImageSink *ximagesink)
       GST_DEBUG ("ximagesink is not using XShm extension");
     }
 #endif /* HAVE_XSHM */
+
+  /* our caps system handles 24/32bpp RGB as big-endian. */
+  if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
+      xcontext->endianness == G_LITTLE_ENDIAN) {
+    xcontext->endianness = G_BIG_ENDIAN;
+    xcontext->visual->red_mask = GULONG_TO_BE (xcontext->visual->red_mask);
+    xcontext->visual->green_mask = GULONG_TO_BE (xcontext->visual->green_mask);
+    xcontext->visual->blue_mask = GULONG_TO_BE (xcontext->visual->blue_mask);
+  }
   
-  if (xcontext->endianness == G_LITTLE_ENDIAN && xcontext->depth == 24)
-    {
-      xcontext->endianness = G_BIG_ENDIAN;
-      xcontext->visual->red_mask = GUINT32_SWAP_LE_BE (xcontext->visual->red_mask);
-      xcontext->visual->green_mask = GUINT32_SWAP_LE_BE (xcontext->visual->green_mask);
-      xcontext->visual->blue_mask = GUINT32_SWAP_LE_BE (xcontext->visual->blue_mask);
-    }
-    
   xcontext->caps = GST_CAPS_NEW ("ximagesink_ximage_caps", "video/x-raw-rgb",
       "bpp",        GST_PROPS_INT (xcontext->bpp),
       "depth",      GST_PROPS_INT (xcontext->depth),
@@ -535,6 +519,7 @@ static GstPadLinkReturn
 gst_ximagesink_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstXImageSink *ximagesink;
+  char *caps_str1, *caps_str2;
 
   ximagesink = GST_XIMAGESINK (gst_pad_get_parent (pad));
 
@@ -546,9 +531,16 @@ gst_ximagesink_sinkconnect (GstPad *pad, GstCaps *caps)
   if (!ximagesink->xcontext)
     return GST_PAD_LINK_DELAYED;
   
-  GST_DEBUG ("sinkconnect %s with %s", gst_caps_to_string(caps),
-             gst_caps_to_string(ximagesink->xcontext->caps));
-  
+  caps_str1 = gst_caps_to_string (ximagesink->xcontext->caps);
+  caps_str2 = gst_caps_to_string (caps);
+
+  GST_DEBUG ("sinkconnect %s with %s", caps_str1, caps_str2);
+
+  if (caps_str1)
+    g_free (caps_str1);
+  if (caps_str2)
+    g_free (caps_str2);
+
   if (!gst_caps_get_int (caps, "width", &(GST_VIDEOSINK_WIDTH (ximagesink))))
     return GST_PAD_LINK_REFUSED;
   if (!gst_caps_get_int (caps, "height", &(GST_VIDEOSINK_HEIGHT (ximagesink))))
@@ -819,14 +811,14 @@ gst_ximagesink_get_bufferpool (GstPad *pad)
 /* Interfaces stuff */
 
 static gboolean
-gst_ximagesink_interface_supported (GstInterface *iface, GType type)
+gst_ximagesink_interface_supported (GstImplementsInterface *iface, GType type)
 {
   g_assert (type == GST_TYPE_NAVIGATION || type == GST_TYPE_X_OVERLAY);
   return TRUE;
 }
 
 static void
-gst_ximagesink_interface_init (GstInterfaceClass *klass)
+gst_ximagesink_interface_init (GstImplementsInterfaceClass *klass)
 {
   klass->supported = gst_ximagesink_interface_supported;
 }
@@ -1076,7 +1068,7 @@ gst_ximagesink_get_type (void)
                                                 "GstXImageSink",
                                                 &ximagesink_info, 0);
       
-      g_type_add_interface_static (ximagesink_type, GST_TYPE_INTERFACE,
+      g_type_add_interface_static (ximagesink_type, GST_TYPE_IMPLEMENTS_INTERFACE,
                                    &iface_info);
       g_type_add_interface_static (ximagesink_type, GST_TYPE_NAVIGATION,
                                    &navigation_info);
@@ -1109,6 +1101,5 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN)
