@@ -47,9 +47,6 @@
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 #include <X11/Xmd.h>
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/Shell.h>
 
 #include "gstxwindow.h"
 #include "gstximage.h"
